<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Module for working with File System Content Renderer and Dynamic Content Renderer.
 * Configuration example:
 *'contentSource' => array(
 *            'class' => 'site.common.modules.contentSource.ContentSourceModule',
 *            'sourceSettings' => array(
 *                'DynamicContentSource' => array(
 *                    'storageFolder' => '/var/www/gPortal/DCR'
 *                ),
 *               'FileSystemContentSource' => array(
 *                    'sourceFolder' => '/var/www/gPortal/FSCR'
 *                ),
 *            ),
 *            'sourceClass' => 'DynamicContentSource'
 *        )
 */
class ContentSourceModule extends CWebModule
{
    /**
     * @var array Content source settings
     */
    public $sourceSettings;
    /**
     * @var string Content source class
     */
    public $sourceClass;

    /**
     * @var string home page
     */
    public $homePage;

    /**
     * This value is used when startpage for some folder doesnt exists
     *
     * @public string
     **/
    public $startPageFallback = 'index.html';

    /**
     * Method of displaying startpage to user. Can take following values:
     * false - redirect user to the page
     * true - (no redirect - just display/include the page without changing the URL) 
     *
     * @public string
     **/
    public $startPageDynamic = true;

    /**
     * Name of content attribute for locale
     * $localizedAttributeName . $locale
     *
     * @public string
     **/
    public $localizedAttributeName = 'startpageurl_';

    /**
     * the number of seconds in which the cached value will expire. 0 means never expire.
     * used for caching content personalisation attributes and modification time
     *
     * @var
     */
    public $cacheTime;
    
    /**
     * an array that specifies the cache-control header of static files, looks like:
     * 
     * array (
     * 	'image/jpeg' => 60 * 60 * 24 * 7,
     * 	'text/css' => 60 * 60 * 24 * 7,
     * 	'application/javascript' => 60 * 60
     *  // etc.
     * )
     * 
     * @var array
     */
    public $staticFileCacheControl;

    /**
     * @var string content source object
     */
    private $_contentSource;

    /**
     * Get object which represents Content Source
     *
     * @return ContentSource
     */
    public function getContentSource()
    {
        if (!$this->_contentSource) {
            $settings = $this->sourceSettings[$this->sourceClass];
            $settings['class'] = $this->sourceClass;
            $this->_contentSource = Yii::createComponent($settings);
            $this->_contentSource->init();
        }
        return $this->_contentSource;
    }


    /**
     * Get static resource content. For example images, css, js
     *
     * @param string $path content path
     *
     * @return Content content
     */
    public function getContent($path)
    {
        return $this->getContentSource()->getContent($path);
    }

    /**
     * Module initial function
     *
     * @return void
     */
    public function init()
    {
        parent::init();
        $this->setImport(
            array(
                'contentSource.components.*'
            )
        );
    }
    
    
    /**
     * Tries to find a browser cache time by path from the config, see $staticFileCacheControl
     * 
     * @param string $filename
     * @return int cachetime
     */
    public function getBrowserCacheTimeByFileName($filename = null)
    {
    	// return -1 if we didn't find any valid cache time in the config
    	$defaultStaticCacheTime = -1;
    	
    	if (!AppHelper::isNullOrEmptyString($filename) && !empty($this->staticFileCacheControl) && is_array($this->staticFileCacheControl)) {
    		
    		// get file mimetype
    		$mimetype = CFileHelper::getMimeTypeByExtension($filename);
    		// if we found a valid mimetype we search if we got a cache time specifies in the browser cache config
		if (isset($this->staticFileCacheControl[$mimetype]) && is_numeric($this->staticFileCacheControl[$mimetype])) {
			$defaultStaticCacheTime = $this->staticFileCacheControl[$mimetype];
		}
    	}
    	
    	return $defaultStaticCacheTime;
    }
}
