<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 *  Class for working with Gentics Content Connector
 */
class PersonalizedContent extends CComponent
{

    /**
     * @var array $folderIds contains folder IDs of content to show
    */
    public $folderIds;

    /**
     * @var string $gccRequestParams contains parameters that should be requested fron GCC
     */
    public $gccRequestParams;

    /**
     * @var string $lang contains current languagecode
     */
    public $lang;

    /**
     * @var string $tag contains content required tag
     */
    public $tag;

    /**
     * array that contains categories tree
     * @var $data
     */

    public $data = array();

    /**
     * @var string $sorting contains sorting expression
     */
    public $sorting;

    /**
     * @var bool $usePersonalisation defines if personalisation should be performed
     */
    public $usePersonalisation = true;

    /**
     * Construction that will load nodes tree for given $folderIds
     *
     * @param array  $folderIds          contains folder IDs of content to show
     * @param bool   $usePersonalisation determines if personalization should be performed
     * @param string $tag                content tag
     *
     * @return \personalizedContent
     */
    public function __construct($folderIds = array(), $usePersonalisation = true, $tag = '', $gccRequestParams = null, $sorting = null)
    {

        //Yii::app()->cache->flush();

        if (!empty($folderIds)) {

            $this->lang = substr(Yii::app()->language, 0, 2);
            $this->gccRequestParams = isset($gccRequestParams)? $gccRequestParams : Yii::app()->getModule('personalizedContent')->gccRequestParams;
            $this->sorting = isset($sorting)? $sorting : Yii::app()->getModule('personalizedContent')->sorting;
            $this->usePersonalisation = $usePersonalisation;
            $this->tag = $tag;

            $this->folderIds = $folderIds;
            sort($this->folderIds);

            $this->_loadData();
            $this->_cleanUp(Yii::app()->user->id);

        } else {
            return false;
        }
    }

    /**
     * Filter tree by some schema
     *
     * @param integer $user_id Id of the user to personalize tree for
     *
     * @return array
     */
    private function _cleanUp($user_id)
    {
        $maxTimestamp = $this->loadMaxUpdatetimestamp();
        $cacheId = 'persContent_PERS_'.md5(implode(',', $this->folderIds).$this->lang.$this->gccRequestParams.$this->sorting.$maxTimestamp.$this->tag);

        if (($data = Yii::app()->cache->get($cacheId)) === false) {
            //get personalisation fields from settings
            $filters = $this->getFiltersList();
            $data = array();

            // apply filter here
            if ($filters && $this->data) {
                foreach ($this->data as $id=>$contentPage) {
                    if (!is_null($this->_unsetNodes($filters, $user_id, $contentPage))) {
                        $data[$id] = $contentPage;
                    }
                }
            }

            $this->data = $data;

            Yii::app()->cache->set($cacheId, $this->data);
        } else {
            $this->data = $data;
        }
    }

    /**
     * Private method that will unset not allowed data
     *
     * @param array $filters contains keys that will be used as filters
     * @param array $user_id contains user permissions
     * @param array $data    points to the array with nodes tree
     *
     * @return array|null
     */
    private function _unsetNodes($filters, $user_id, $data)
    {
        /*  check permissions for current node */
        $persAttributes = array();
        foreach ($filters as $filter) {
            if (!empty($data['attributes'][$filter])) {
                $persAttributes = array_merge($persAttributes, $data['attributes'][$filter]);
            }
        }

        //check if page has allowed startpageurl and exlude the branch if it does not.
        $excludeURLs = Yii::app()->getModule('personalizedContent')->excludeURLs;
        if ((isset($data['attributes']['url']) && in_array($data['attributes']['url'], $excludeURLs)) || !isset($data['attributes']['url'])) {
            return null;
        }

        // Call function that will check permissions
        if ($this->usePersonalisation) {
            if (!Yii::app()->getModule('personalisation')->rule->checkAccess($user_id, $persAttributes)) {
                /* if user does not have permissions - unset the node */
                return null;
            }
        }

        return $data;
    }

    /**
     * Filter tree by some schema
     *
     * @return array|bool
     */
    public function getFiltersList()
    {
        if (Yii::app()->getModule('contentSource')->contentSource->usePersonalisation) {
            $filters = Yii::app()->getModule('contentSource')->contentSource->personalisationFields;
            return $filters;
        }

        return false;
    }

    /**
     * Load required tree data from cache. If not exists request Gentics Content Connector API for a new tree.
     *
     * @return array
     */
    private function _loadData()
    {
        $data = array();
        $filtersList = '';

        $this->gccRequestParams = rtrim($this->gccRequestParams, ',');
        $requertFields = explode(',', $this->gccRequestParams);

        $requertFields[] = 'updatetimestamp';
        $requertFields[] = 'name';
        $requertFields[] = 'url';

        $requertFields = array_unique($requertFields);

        // get max timestamp for given folderIds
        $maxTimestamp = $this->loadMaxUpdatetimestamp();

        //create cache key that includes folderIds+language+gccRequestString+timestamp of last updated folder;
        $cacheId = 'persContent_'.md5(implode(',', $this->folderIds).$this->lang.$this->gccRequestParams.$maxTimestamp.$this->tag);

        //Load data
        if (($data = Yii::app()->cache->get($cacheId)) === false) {
            $params = array(
                'filter' => 'object.obj_type==10007 AND object.hideinoverview != 1 AND object.languagecode=="'.$this->lang.'" AND (object.folder_id=="'.implode('" OR object.folder_id=="', $this->folderIds).'")',
                'type' => 'php',
                'sorting' => $this->sorting,
            );

            if (!empty($this->tag)) {
                $params['filter'] .= 'AND object.tagging LIKE "*'.$this->tag.'*"';
            }

            $params = Yii::app()->getComponent('repositoryApi')->addAdditionalParameters($params, 'contentRepositorySearch');

            $data = Yii::app()->repositoryApi->request(
                $params,
                CMap::mergeArray($this->getFiltersList(), $requertFields)
            );
            $data = unserialize($data);

            if (isset($data['status']) && $data['status'] == 'ok') {
                unset($data['status']);
            } else {
                $data = array();
            }

            Yii::app()->cache->set($cacheId, $data);
        }

        $this->data = $data;
    }

    /**
     * extracts max updatetimestamp from array of CMS objects
     *
     * @return int|bool
     */
    public function loadMaxUpdatetimestamp()
    {
        $timestampCacheId = 'persContent_UTS_'.md5(implode(',', $this->folderIds).$this->lang.$this->gccRequestParams.$this->sorting.$this->tag);

        //try to get $maxTimestamp from cache
        if (($maxTimestamp = Yii::app()->cache->get($timestampCacheId)) === false) {
            $params = array(
                'filter' => 'object.obj_type==10007 AND object.hideinoverview != 1 AND object.languagecode=="'.$this->lang.'" AND (object.folder_id=="'.implode('" OR object.folder_id=="', $this->folderIds).'")',
                'type' => 'php',
                'sorting' => $this->sorting,
            );
            if (!empty($this->tag)) {
                $params['filter'] .= 'AND object.tagging LIKE "*'.$this->tag.'*"';
            }

            $params = Yii::app()->getComponent('repositoryApi')->addAdditionalParameters($params, 'contentRepositorySearch');

            $timestamps = Yii::app()->repositoryApi->request(
                $params,
                array('updatetimestamp')
            );
            $timestamps = unserialize($timestamps);

            if (isset($timestamps['status']) && $timestamps['status']=='ok') {
                unset($timestamps['status']);
                $maxTimestamp = 0;
                foreach ($timestamps as $id=>$contentFolder) {
                    if (isset($contentFolder['attributes']['updatetimestamp']) && $contentFolder['attributes']['updatetimestamp'] > $maxTimestamp) {
                        $maxTimestamp = $contentFolder['attributes']['updatetimestamp'];
                    }
                }

                Yii::app()->cache->set($timestampCacheId, $maxTimestamp, Yii::app()->getModule('personalizedContent')->cacheTime);
            } else {
                $maxTimestamp = false;
            }
        }
        return $maxTimestamp;
    }
}
